# -*- coding: utf-8 -*-
import logging

from pyramid.httpexceptions import HTTPFound

log = logging.getLogger(__name__)


SETTINGS_PREFIX = 'security.google_login.'


class AuthFailed(Exception):
    pass


def includeme(config):
    log.info("Add pyramid_google_login")

    config.include('pyramid_mako')

    config.add_route('auth_signin', '/auth/signin')
    config.add_route('auth_signin_redirect', '/auth/signin_redirect')
    config.add_route('auth_callback', '/auth/oauth2callback')
    config.add_route('auth_logout', '/auth/logout')

    config.add_static_view('static', 'pyramid_google_login:static')

    config.scan()


def redirect_to_signin(request, message=None, url=None, headers=None):
    """ Redirect to the sign in page with message and next url """
    query = {}
    if message is not None:
        query['message'] = message
    if url is not None:
        query['url'] = url
    url = request.route_url('auth_signin', _query=query)
    raise HTTPFound(location=url, headers=headers)



import logging

from pyramid.view import view_config
from pyramid.security import (remember, forget, NO_PERMISSION_REQUIRED)
from pyramid.httpexceptions import HTTPFound

from pyramid_google_login import SETTINGS_PREFIX
from pyramid_google_login import redirect_to_signin
from pyramid_google_login import AuthFailed
from pyramid_google_login.google_oauth2 import (build_authorize_url,
                                                exchange_token_from_code,
                                                get_userinfo_from_token,
                                                check_hosted_domain_user,
                                                get_principal_from_userinfo,
                                                encode_state,
                                                decode_state,
                                                )

log = logging.getLogger(__name__)


@view_config(route_name='auth_signin',
             permission=NO_PERMISSION_REQUIRED,
             renderer='pyramid_google_login:templates/signin.mako')
def signin(request):
    settings = request.registry.settings
    signin_banner = settings.get(SETTINGS_PREFIX + 'signin_banner')
    hosted_domain = settings.get(SETTINGS_PREFIX + 'hosted_domain')
    signin_advice = settings.get(SETTINGS_PREFIX + 'signin_advice')
    message = request.params.get('message')

    if 'url' in request.params:
        query = {'url': request.params['url']}
        url = request.route_url('auth_signin_redirect', _query=query)
    else:
        url = request.route_url('auth_signin_redirect')

    return {'signin_redirect_url': url,
            'message': message,
            'signin_banner': signin_banner,
            'signin_advice': signin_advice,
            'hosted_domain': hosted_domain,
            }


@view_config(route_name='auth_signin_redirect',
             permission=NO_PERMISSION_REQUIRED,)
def signin_redirect(request):
    state_params = {}
    if 'url' in request.params:
        state_params['url'] = request.params['url']

    state = encode_state(state_params)
    try:
        redirect_uri = build_authorize_url(request, state)
    except AuthFailed as err:
        log.warning("Google Login failed (%s)", err)
        redirect_to_signin(request, "Google Login failed (%s)" % err)

    return HTTPFound(location=redirect_uri)


@view_config(route_name='auth_callback',
             permission=NO_PERMISSION_REQUIRED)
def callback(request):
    settings = request.registry.settings
    landing_url = settings.get(SETTINGS_PREFIX + 'landing_url', '/')
    max_age = int(settings.get(SETTINGS_PREFIX + 'max_age', 24 * 3600))

    try:
        access_token = exchange_token_from_code(request)
        userinfo = get_userinfo_from_token(access_token)
        check_hosted_domain_user(request, userinfo)
        principal = get_principal_from_userinfo(request, userinfo)

    except AuthFailed as err:
        log.warning("Google Login failed (%s)", err)
        redirect_to_signin(request, "Google Login failed (%s)" % err)

    except Exception as err:
        log.warning("Google Login failed (%s)", err)
        # Protect against leaking critical information like client_secret
        redirect_to_signin(request, "Google Login failed (unkown)")

    # Find the redirect url (fail-safe, the authentication is more important)
    try:
        state_params = decode_state(request.params['state'])
        url = state_params['url'][0]
    except:
        url = landing_url

    headers = remember(request, principal=principal, max_age=max_age)
    return HTTPFound(location=url, headers=headers)


@view_config(route_name='auth_logout')
def logout(request):
    headers = forget(request)
    redirect_to_signin(request, "You are logged out!", headers=headers)



import logging
import urllib
import urlparse

import requests
from requests.exceptions import RequestException

from pyramid_google_login import SETTINGS_PREFIX
from pyramid_google_login import AuthFailed

log = logging.getLogger(__name__)


GOOGLE_AUTHORIZE = "https://accounts.google.com/o/oauth2/auth"
GOOGLE_TOKEN = "https://accounts.google.com/o/oauth2/token"
GOOGLE_USERINFO = "https://www.googleapis.com/oauth2/v2/userinfo"


def encode_state(params):
    return urllib.urlencode(params)


def decode_state(state):
    return urlparse.parse_qs(state)


def build_authorize_url(request, state):
    settings = request.registry.settings
    hosted_domain = settings.get(SETTINGS_PREFIX + 'hosted_domain')
    try:
        client_id = settings[SETTINGS_PREFIX + 'client_id']
    except:
        raise AuthFailed("Missing settings")

    params = {
        "response_type": "code",
        "client_id": client_id,
        "redirect_uri": request.route_url("auth_callback"),
        "scope": "email",
        "state": state,
    }

    if hosted_domain is not None:
        params['hd'] = hosted_domain

    authorize_url = "%s?%s" % (GOOGLE_AUTHORIZE, urllib.urlencode(params))

    return authorize_url


def exchange_token_from_code(request):
    if 'error' in request.params:
        raise AuthFailed("Error from Google (%s)" % request.params['error'])

    try:
        code = request.params['code']
    except KeyError as err:
        raise AuthFailed("No authorization code from Google")

    settings = request.registry.settings
    try:
        client_id = settings[SETTINGS_PREFIX + 'client_id']
        client_secret = settings[SETTINGS_PREFIX + 'client_secret']
    except KeyError as err:
        raise AuthFailed("Missing settings")

    params = {
        "code": code,
        "client_id": client_id,
        "client_secret": client_secret,
        "redirect_uri": request.route_url("auth_callback"),
        "grant_type": "authorization_code",
    }

    try:
        resp = requests.post(GOOGLE_TOKEN, data=params)
        resp.raise_for_status()
        oauth2_response = resp.json()
    except RequestException as err:
        raise AuthFailed("Failed to get token from Google (%s)" % err)
    except Exception as err:
        log.warning("Unkown error while calling token endpoint",
                    exc_info=True)
        raise AuthFailed("Failed to get token from Google (unkown error)")

    try:
        access_token = oauth2_response['access_token']
    except KeyError:
        raise AuthFailed("No access_token in response from Google")

    return access_token


def get_userinfo_from_token(access_token):
    try:
        resp = requests.get(GOOGLE_USERINFO,
                            params={'access_token': access_token})
        resp.raise_for_status()
        return resp.json()
    except Exception:
        log.warning("Unkown error calling userinfo endpoint",
                    exc_info=True)
        raise AuthFailed("Failed to get userinfo from Google")


def check_hosted_domain_user(request, userinfo):
    settings = request.registry.settings
    hosted_domain = settings.get(SETTINGS_PREFIX + 'hosted_domain')

    if hosted_domain is None:
        return

    try:
        user_hosted_domain = userinfo['hd']
    except KeyError:
        raise AuthFailed("Missing hd field from Google userinfo")

    if hosted_domain != user_hosted_domain:
        raise AuthFailed("You logged in with an unkown domain "
                         "(%s rather than %s)" % (user_hosted_domain,
                                                  hosted_domain))


def get_principal_from_userinfo(request, userinfo):
    user_id_field = request.registry.settings.get(
        SETTINGS_PREFIX + 'user_id_field',
        'email')

    try:
        principal = userinfo[user_id_field]
    except:
        raise AuthFailed("Missing principal field from Google userinfo")

    return principal
